@@ -33,6 +33,10 @@ Copyright (C) 1994 Steen Lumholt.
 #include <windows.h>
 #endif
 
+#ifdef macintosh
+#define MAC_TCL
+#endif
+
 /* Allow using this code in Python 2.[12] */
 #ifndef PyDoc_STRVAR
 #define PyDoc_STRVAR(name,str) static char name[] = str
@@ -92,22 +96,20 @@ Copyright (C) 1994 Steen Lumholt.
 #error "unsupported Tcl configuration"
 #endif
 
-#if !(defined(MS_WINDOWS) || defined(__CYGWIN__))
+#if defined(macintosh)
+/* Sigh, we have to include this to get at the tcl qd pointer */
+#include <tkMac.h>
+/* And this one we need to clear the menu bar */
+#include <Menus.h>
+#endif
+
+#if !(defined(MS_WINDOWS) || defined(__CYGWIN__) || defined(macintosh))
+/* Mac has it, but it doesn't really work:-( */
 #define HAVE_CREATEFILEHANDLER
 #endif
 
 #ifdef HAVE_CREATEFILEHANDLER
 
-/* This bit is to ensure that TCL_UNIX_FD is defined and doesn't interfere
-   with the proper calculation of FHANDLETYPE == TCL_UNIX_FD below. */
-#ifndef TCL_UNIX_FD
-#  ifdef TCL_WIN_SOCKET
-#    define TCL_UNIX_FD (! TCL_WIN_SOCKET)
-#  else
-#    define TCL_UNIX_FD 1
-#  endif
-#endif
-
 /* Tcl_CreateFileHandler() changed several times; these macros deal with the
    messiness.  In Tcl 8.0 and later, it is not available on Windows (and on
    Unix, only because Jack added it back); when available on Windows, it only
@@ -236,6 +238,29 @@ static PyThreadState *tcl_tstate = NULL;
 
 #endif
 
+#ifdef macintosh
+
+/*
+** Additional cruft needed by Tcl/Tk on the Mac.
+** This is for Tcl 7.5 and Tk 4.1 (patch release 1).
+*/
+
+/* ckfree() expects a char* */
+#define FREECAST (char *)
+
+#include <Events.h> /* For EventRecord */
+
+typedef int (*TclMacConvertEventPtr) (EventRecord *eventPtr);
+void Tcl_MacSetEventProc(TclMacConvertEventPtr procPtr);
+int TkMacConvertEvent(EventRecord *eventPtr);
+
+static int PyMacConvertEvent(EventRecord *eventPtr);
+
+#include <SIOUX.h>
+extern int SIOUXIsAppWindow(WindowPtr);
+
+#endif /* macintosh */
+
 #ifndef FREECAST
 #define FREECAST (char *)
 #endif
@@ -556,19 +581,15 @@ int
 Tcl_AppInit(Tcl_Interp *interp)
 {
 	Tk_Window main;
-	const char * _tkinter_skip_tk_init;
 
+	main = Tk_MainWindow(interp);
 	if (Tcl_Init(interp) == TCL_ERROR) {
 		PySys_WriteStderr("Tcl_Init error: %s\n", Tcl_GetStringResult(interp));
 		return TCL_ERROR;
 	}
-	_tkinter_skip_tk_init =	Tcl_GetVar(interp, "_tkinter_skip_tk_init", TCL_GLOBAL_ONLY);
-	if (_tkinter_skip_tk_init == NULL || strcmp(_tkinter_skip_tk_init, "1")	!= 0) {
-		main = Tk_MainWindow(interp);
-		if (Tk_Init(interp) == TCL_ERROR) {
-			PySys_WriteStderr("Tk_Init error: %s\n", Tcl_GetStringResult(interp));
-			return TCL_ERROR;
-		}
+	if (Tk_Init(interp) == TCL_ERROR) {
+		PySys_WriteStderr("Tk_Init error: %s\n", Tcl_GetStringResult(interp));
+		return TCL_ERROR;
 	}
 	return TCL_OK;
 }
@@ -586,7 +607,7 @@ static void DisableEventHook(void); /* Forward */
 
 static TkappObject *
 Tkapp_New(char *screenName, char *baseName, char *className,
-	  int interactive, int wantobjects, int	wantTk, int sync, char *use)
+	  int interactive, int wantobjects)
 {
 	TkappObject *v;
 	char *argv0;
@@ -625,6 +646,12 @@ Tkapp_New(char *screenName, char *baseName, char *className,
 	v->ProcBodyType = Tcl_GetObjType("procbody");
 	v->StringType = Tcl_GetObjType("string");
 
+#if defined(macintosh)
+	/* This seems to be needed */
+	ClearMenuBar();
+	TkMacInitMenus(v->interp);
+#endif
+
 	/* Delete the 'exit' command, which can screw things up */
 	Tcl_DeleteCommand(v->interp, "exit");
 
@@ -646,51 +673,13 @@ Tkapp_New(char *screenName, char *baseName, char *className,
 	}
 
 	strcpy(argv0, className);
-	if (isupper(Py_CHARMASK(argv0[0])))
-		argv0[0] = tolower(Py_CHARMASK(argv0[0]));
+	if (isupper((int)(argv0[0])))
+		argv0[0] = tolower(argv0[0]);
 	Tcl_SetVar(v->interp, "argv0", argv0, TCL_GLOBAL_ONLY);
 	ckfree(argv0);
 
-	if (! wantTk) {
-	    Tcl_SetVar(v->interp, "_tkinter_skip_tk_init", "1",	TCL_GLOBAL_ONLY);
-	}
-
-	/* some initial arguments need to be in argv */
-	if (sync || use) {
-		char *args;
-		int len = 0;
-
-		if (sync)
-			len += sizeof "-sync";
-		if (use)
-			len += strlen(use) + sizeof "-use ";
-
-		args = (char*)ckalloc(len);
-		if (!args) {
-			PyErr_NoMemory();
-			Py_DECREF(v);
-			return NULL;
-		}
-
-		args[0] = '\0';
-		if (sync)
-			strcat(args, "-sync");
-		if (use) {
-			if (sync)
-				strcat(args, " ");
-			strcat(args, "-use ");
-			strcat(args, use);
-		}
-
-		Tcl_SetVar(v->interp, "argv", args, TCL_GLOBAL_ONLY);
-		ckfree(args);
-	}
-
-	if (Tcl_AppInit(v->interp) != TCL_OK) {
-		PyObject *result = Tkinter_Error((PyObject *)v);
-		Py_DECREF((PyObject *)v);
-		return (TkappObject *)result;
-	}
+	if (Tcl_AppInit(v->interp) != TCL_OK)
+		return (TkappObject *)Tkinter_Error((PyObject *)v);
 
 	EnableEventHook();
 
@@ -818,8 +807,8 @@ static PyObject *
 PyTclObject_repr(PyTclObject *self)
 {
 	char buf[50];
-	PyOS_snprintf(buf, 50, "<%s object at %p>",
-		      self->value->typePtr->name, self->value);
+	PyOS_snprintf(buf, 50, "<%s object at 0x%.8x>",
+		      self->value->typePtr->name, (int)self->value);
 	return PyString_FromString(buf);
 }
 
@@ -851,10 +840,8 @@ static PyGetSetDef PyTclObject_getsetlist[] = {
 };
 
 static PyMethodDef PyTclObject_methods[] = {
-#ifdef Py_USING_UNICODE
 	{"__unicode__",	(PyCFunction)PyTclObject_unicode, METH_NOARGS,
 	PyTclObject_unicode__doc__},
-#endif
 	{0}
 };
 
@@ -932,13 +919,12 @@ AsObj(PyObject *value)
 #ifdef Py_USING_UNICODE
 	else if (PyUnicode_Check(value)) {
 		Py_UNICODE *inbuf = PyUnicode_AS_UNICODE(value);
-		Py_ssize_t size = PyUnicode_GET_SIZE(value);
+		int size = PyUnicode_GET_SIZE(value);
 		/* This #ifdef assumes that Tcl uses UCS-2.
 		   See TCL_UTF_MAX test above. */
 #if defined(Py_UNICODE_WIDE) && TCL_UTF_MAX == 3
 		Tcl_UniChar *outbuf;
-		Py_ssize_t i;
-		assert(size < size * sizeof(Tcl_UniChar));
+		int i;
 		outbuf = (Tcl_UniChar*)ckalloc(size * sizeof(Tcl_UniChar));
 		if (!outbuf) {
 			PyErr_NoMemory();
@@ -1007,7 +993,7 @@ FromObj(PyObject* tkapp, Tcl_Obj *value)
 			}
 		}
 #else
-		result = PyString_FromStringAndSize(value->bytes, value->length);
+		res = PyString_FromStringAndSize(value->bytes, value->length);
 #endif
 		return result;
 	}
@@ -1275,13 +1261,13 @@ Tkapp_CallProc(Tkapp_CallEvent *e, int flags)
       and perform processing there. */
 
 static PyObject *
-Tkapp_Call(PyObject *selfptr, PyObject *args)
+Tkapp_Call(PyObject *_self, PyObject *args)
 {
 	Tcl_Obj *objStore[ARGSZ];
 	Tcl_Obj **objv = NULL;
 	int objc, i;
 	PyObject *res = NULL;
-	TkappObject *self = (TkappObject*)selfptr;
+	TkappObject *self = (TkappObject*)_self;
 	/* Could add TCL_EVAL_GLOBAL if wrapped by GlobalCall... */
 	int flags = TCL_EVAL_DIRECT;
 
@@ -1327,7 +1313,7 @@ Tkapp_Call(PyObject *selfptr, PyObject *args)
 		ENTER_OVERLAP
 
 		if (i == TCL_ERROR)
-			Tkinter_Error(selfptr);
+			Tkinter_Error(_self);
 		else
 			res = Tkapp_CallResult(self);
 
@@ -1543,12 +1529,12 @@ var_proc(VarEvent* ev, int flags)
 }
 
 static PyObject*
-var_invoke(EventFunc func, PyObject *selfptr, PyObject *args, int flags)
+var_invoke(EventFunc func, PyObject *_self, PyObject *args, int flags)
 {
-	TkappObject *self = (TkappObject*)selfptr;
+	TkappObject *self = (TkappObject*)_self;
 #ifdef WITH_THREAD
 	if (self->threaded && self->thread_id != Tcl_GetCurrentThread()) {
-		TkappObject *self = (TkappObject*)selfptr;
+		TkappObject *self = (TkappObject*)_self;
 		VarEvent *ev;
 		PyObject *res, *exc_type, *exc_val;
 		
@@ -1560,7 +1546,7 @@ var_invoke(EventFunc func, PyObject *selfptr, PyObject *args, int flags)
 
 		ev = (VarEvent*)ckalloc(sizeof(VarEvent));
 
-		ev->self = selfptr;
+		ev->self = _self;
 		ev->args = args;
 		ev->flags = flags;
 		ev->func = func;
@@ -1580,7 +1566,7 @@ var_invoke(EventFunc func, PyObject *selfptr, PyObject *args, int flags)
 	}
 #endif
         /* Tcl is not threaded, or this is the interpreter thread. */
-	return func(selfptr, args, flags);
+	return func(_self, args, flags);
 }
 
 static PyObject *
@@ -2080,9 +2066,9 @@ Tkapp_CommandProc(CommandEvent *ev, int flags)
 }
 
 static PyObject *
-Tkapp_CreateCommand(PyObject *selfptr, PyObject *args)
+Tkapp_CreateCommand(PyObject *_self, PyObject *args)
 {
-	TkappObject *self = (TkappObject*)selfptr;
+	TkappObject *self = (TkappObject*)_self;
 	PythonCmd_ClientData *data;
 	char *cmdName;
 	PyObject *func;
@@ -2104,9 +2090,9 @@ Tkapp_CreateCommand(PyObject *selfptr, PyObject *args)
 	data = PyMem_NEW(PythonCmd_ClientData, 1);
 	if (!data)
 		return PyErr_NoMemory();
-	Py_INCREF(self);
-	Py_INCREF(func);
-	data->self = selfptr;
+	Py_XINCREF(self);
+	Py_XINCREF(func);
+	data->self = _self;
 	data->func = func;
 	
 	if (self->threaded && self->thread_id != Tcl_GetCurrentThread()) {
@@ -2140,9 +2126,9 @@ Tkapp_CreateCommand(PyObject *selfptr, PyObject *args)
 
 
 static PyObject *
-Tkapp_DeleteCommand(PyObject *selfptr, PyObject *args)
+Tkapp_DeleteCommand(PyObject *_self, PyObject *args)
 {
-	TkappObject *self = (TkappObject*)selfptr;
+	TkappObject *self = (TkappObject*)_self;
 	char *cmdName;
 	int err;
 
@@ -2493,10 +2479,8 @@ Tkapp_CreateTimerHandler(PyObject *self, PyObject *args)
 	}
 
 	v = Tktt_New(func);
-	if (v) {
-		v->token = Tcl_CreateTimerHandler(milliseconds, TimerHandler,
-						  (ClientData)v);
-	}
+	v->token = Tcl_CreateTimerHandler(milliseconds, TimerHandler,
+					  (ClientData)v);
 
 	return (PyObject *) v;
 }
@@ -2505,10 +2489,10 @@ Tkapp_CreateTimerHandler(PyObject *self, PyObject *args)
 /** Event Loop **/
 
 static PyObject *
-Tkapp_MainLoop(PyObject *selfptr, PyObject *args)
+Tkapp_MainLoop(PyObject *_self, PyObject *args)
 {
 	int threshold = 0;
-	TkappObject *self = (TkappObject*)selfptr;
+	TkappObject *self = (TkappObject*)_self;
 #ifdef WITH_THREAD
 	PyThreadState *tstate = PyThreadState_Get();
 #endif
@@ -2619,52 +2603,6 @@ Tkapp_InterpAddr(PyObject *self, PyObject *args)
 	return PyInt_FromLong((long)Tkapp_Interp(self));
 }
 
-static PyObject	*
-Tkapp_TkInit(PyObject *self, PyObject *args)
-{
-	static int has_failed;
-	Tcl_Interp *interp = Tkapp_Interp(self);
-	Tk_Window main_window;
-	const char * _tk_exists = NULL;
-	int err;
-	main_window = Tk_MainWindow(interp);
-
-	/* In all current versions of Tk (including 8.4.13), Tk_Init
-	   deadlocks on the second call when the first call failed.
-	   To avoid the deadlock, we just refuse the second call through
-	   a static variable. */
-	if (has_failed) {
-		PyErr_SetString(Tkinter_TclError, 
-				"Calling Tk_Init again after a previous call failed might deadlock");
-		return NULL;
-	}
-	   
-	/* We want to guard against calling Tk_Init() multiple times */
-	CHECK_TCL_APPARTMENT;
-	ENTER_TCL
-	err = Tcl_Eval(Tkapp_Interp(self), "info exists	tk_version");
-	ENTER_OVERLAP
-	if (err == TCL_ERROR) {
-		/* This sets an exception, but we cannot return right
-		   away because we need to exit the overlap first. */
-		Tkinter_Error(self);
-	} else {
-		_tk_exists = Tkapp_Result(self);
-	}
-	LEAVE_OVERLAP_TCL
-	if (err == TCL_ERROR) {
-		return NULL;
-	}
-	if (_tk_exists == NULL || strcmp(_tk_exists, "1") != 0)	{
-		if (Tk_Init(interp)	== TCL_ERROR) {
-		        PyErr_SetString(Tkinter_TclError, Tcl_GetStringResult(Tkapp_Interp(self)));
-			has_failed = 1;
-			return NULL;
-		}
-	}
-	Py_INCREF(Py_None);
-	return Py_None;
-}
 
 static PyObject *
 Tkapp_WantObjects(PyObject *self, PyObject *args)
@@ -2732,7 +2670,6 @@ static PyMethodDef Tkapp_methods[] =
 	{"dooneevent", 	       Tkapp_DoOneEvent, METH_VARARGS},
 	{"quit", 	       Tkapp_Quit, METH_VARARGS},
 	{"interpaddr",         Tkapp_InterpAddr, METH_VARARGS},
-	{"loadtk",	       Tkapp_TkInit, METH_NOARGS},
 	{NULL, 		       NULL}
 };
 
@@ -2897,9 +2834,6 @@ Tkinter_Create(PyObject *self, PyObject *args)
 	char *className = NULL;
 	int interactive = 0;
 	int wantobjects = 0;
-	int wantTk = 1;	/* If false, then Tk_Init() doesn't get	called */
-	int sync = 0; /* pass -sync to wish */
-	char *use = NULL; /* pass -use to wish */
 
 	baseName = strrchr(Py_GetProgramName(), '/');
 	if (baseName != NULL)
@@ -2908,15 +2842,13 @@ Tkinter_Create(PyObject *self, PyObject *args)
 		baseName = Py_GetProgramName();
 	className = "Tk";
   
-	if (!PyArg_ParseTuple(args, "|zssiiiiz:create",
+	if (!PyArg_ParseTuple(args, "|zssii:create",
 			      &screenName, &baseName, &className,
-			      &interactive, &wantobjects, &wantTk,
-			      &sync, &use))
+			      &interactive, &wantobjects))
 		return NULL;
 
 	return (PyObject *) Tkapp_New(screenName, baseName, className, 
-				      interactive, wantobjects,	wantTk,
-				      sync, use);
+				      interactive, wantobjects);
 }
 
 static PyObject *
@@ -3106,8 +3038,6 @@ init_tkinter(void)
 #endif
 
 	m = Py_InitModule("_tkinter", moduleMethods);
-	if (m == NULL)
-		return;
 
 	d = PyModule_GetDict(m);
 	Tkinter_TclError = PyErr_NewException("_tkinter.TclError", NULL, NULL);
@@ -3162,4 +3092,112 @@ init_tkinter(void)
 	Py_AtExit(Tcl_Finalize);
 #endif
 
+#ifdef macintosh
+	/*
+	** Part of this code is stolen from MacintoshInit in tkMacAppInit.
+	** Most of the initializations in that routine (toolbox init calls and
+	** such) have already been done for us, so we only need these.
+	*/
+	tcl_macQdPtr = &qd;
+
+	Tcl_MacSetEventProc(PyMacConvertEvent);
+#if GENERATINGCFM
+	mac_addlibresources();
+#endif /* GENERATINGCFM */
+#endif /* macintosh */
 }
+
+
+
+#ifdef macintosh
+
+/*
+** Anyone who embeds Tcl/Tk on the Mac must define panic().
+*/
+
+void
+panic(char * format, ...)
+{
+	va_list varg;
+	
+	va_start(varg, format);
+	
+	vfprintf(stderr, format, varg);
+	(void) fflush(stderr);
+	
+	va_end(varg);
+
+	Py_FatalError("Tcl/Tk panic");
+}
+
+/*
+** Pass events to SIOUX before passing them to Tk.
+*/
+
+static int
+PyMacConvertEvent(EventRecord *eventPtr)
+{
+	WindowPtr frontwin;
+	/*
+	** Sioux eats too many events, so we don't pass it everything.  We
+	** always pass update events to Sioux, and we only pass other events if
+	** the Sioux window is frontmost. This means that Tk menus don't work
+	** in that case, but at least we can scroll the sioux window.
+	** Note that the SIOUXIsAppWindow() routine we use here is not really
+	** part of the external interface of Sioux...
+	*/
+	frontwin = FrontWindow();
+	if ( eventPtr->what == updateEvt || SIOUXIsAppWindow(frontwin) ) {
+		if (SIOUXHandleOneEvent(eventPtr))
+			return 0; /* Nothing happened to the Tcl event queue */
+	}
+	return TkMacConvertEvent(eventPtr);
+}
+
+#if GENERATINGCFM
+
+/*
+** Additional Mac specific code for dealing with shared libraries.
+*/
+
+#include <Resources.h>
+#include <CodeFragments.h>
+
+static int loaded_from_shlib = 0;
+static FSSpec library_fss;
+
+/*
+** If this module is dynamically loaded the following routine should
+** be the init routine. It takes care of adding the shared library to
+** the resource-file chain, so that the tk routines can find their
+** resources.
+*/
+OSErr pascal
+init_tkinter_shlib(CFragInitBlockPtr data)
+{
+	__initialize();
+	if ( data == nil ) return noErr;
+	if ( data->fragLocator.where == kDataForkCFragLocator ) {
+		library_fss = *data->fragLocator.u.onDisk.fileSpec;
+		loaded_from_shlib = 1;
+	} else if ( data->fragLocator.where == kResourceCFragLocator ) {
+		library_fss = *data->fragLocator.u.inSegs.fileSpec;
+		loaded_from_shlib = 1;
+	}
+	return noErr;
+}
+
+/*
+** Insert the library resources into the search path. Put them after
+** the resources from the application. Again, we ignore errors.
+*/
+static
+mac_addlibresources(void)
+{
+	if ( !loaded_from_shlib ) 
+		return;
+	(void)FSpOpenResFile(&library_fss, fsRdPerm);
+}
+
+#endif /* GENERATINGCFM */
+#endif /* macintosh */
